%{
/*
 * Copyright (c) 2003-2006 Erez Zadok
 * Copyright (c) 2003-2006 Stony Brook University
 * Copyright (c) 2003-2006 The Research Foundation of State University of New York
 * Copyright (c) 2004-2006 Nikolai Joukov
 * Copyright (c) 2003-2005 Akshat Aranya
 * Copyright (c) 2006-2006 Gopala Surya
 */
	/*
	 * Lexer for tracefs configuration file
	 */
#include <string.h>
#include "confparser.h"

int comment();
#define _DECLARE_SYSCALL_TABLE_
#define NEED_DESCRIPTIVE_MESSAGES
#include <sys/types.h>
#include "../messages.h"
#define _VERBOSITY_CONSTANTS_
#include "../common.h"
#include "../syscalls.h"
%}

%%

"/*"		{ comment(); }
AND      	{ yylval.i = AND; return AND; }
OR       	{ yylval.i = OR; return OR; }
NOT      	{ yylval.i = NOT; return NOT; }
filename 	{ yylval.i = FILENAME; return FILENAME; }
async   	{ yylval.i = ASYNC; return ASYNC; }
count   	{ yylval.i = COUNT; return COUNT; }
cksum           { yylval.i = CKSUM; return CKSUM; }
compress        { yylval.i = COMPRESS; return COMPRESS; }
cuid     	{ yylval.i = CUID; return CUID; }
cgid     	{ yylval.i = CGID; return CGID; }
ext      	{ yylval.i = EXT; return EXT; }
true     	{ yylval.i = TRUE; return TRUE; }
false    	{ yylval.i = FALSE; return FALSE; }
ops      	{ yylval.i = OPS; return OPS; }
sync_writes    	{ yylval.i = SYNC_WRITES; return SYNC_WRITES; }
hostname 	{ return HOSTNAME; }
port     	{ return PORT; }
inode    	{ yylval.i = INO; return INO; }
syscalls 	{ return SYSCALLS; }
time     	{ return TIME; }
sid      	{ yylval.i = SID; return SID; }
pid      	{ yylval.i = PID; return PID; }
pgrp      	{ yylval.i = PGRP; return PGRP; }
buffer          { yylval.i = BUFFER; return BUFFER; }
buf      	{ yylval.i = BUF; return BUF; }
pname    	{ yylval.i = PNAME; return PNAME; }
stream   	{ yylval.i = STREAM; return STREAM; }
aggregate	{ yylval.i = AGGREGATE; return AGGREGATE; }
encrypt         { yylval.i = ENCRYPT; return ENCRYPT; }
0[xX][a-fA-F0-9]+ { yylval.c = yytext; return HEXSTR; }
[0-9]+	{ yylval.i = atoi(yytext); return INT; }

\$[_a-zA-Z][_a-zA-Z0-9]* {
			char *p = getenv(yytext + 1);
			if (p == NULL) {
				p = "";
			}
			yylval.c = p;
			return QSTR; /* returning a pointer from environment. Caller must dup the string */
		}

\"[^\n\"]*[\n\"] { yylval.c = yytext + 1; yylval.c[strlen(yylval.c) - 1] = '\0'; return QSTR; /* returning a pointer to yytext here. Be CAREFUL to copy the text before you use it */}
STR_[A-Z_]+        {
		      int i;
		      for (i = 0; _verbosity_opts[i].name != NULL; i++) {
                          if (strcmp(yytext, _verbosity_opts[i].name) == 0) {
				yylval.i = _verbosity_opts[i].id;
				return STREAM_VERBOSITY_OP;
                          }
                     }
                     return STR;
                  }

NR_[_a-zA-Z0-9_]+ {
			int i;
			for (i = 0;  __syscalls[i].name != NULL; i++) {
				if (strcmp(yytext+3, __syscalls[i].name) == 0) {
					yylval.i = __syscalls[i].id - SYSCALL_OFFSET;
					return SYSCALL_ID;
				}
			}
			return STR;
		  }
[_a-zA-Z]+        {
                      int i;
                      for (i = 0; msg_desc[i].str_type != NULL; i++) {
                          if (msg_desc[i].msg_type < OP_MAX && strcmp(yytext, msg_desc[i].str_type) == 0) {
                              yylval.i = msg_desc[i].msg_type;
                              return OP_TYPE;
                          }
                      }
                      return STR;
                  }
[ \t\n]           { /*ignore*/ }
. { return yytext[0]; }

%%

int yywrap()
{
	return 1;
}

int comment()
{
	char c, c1;

loop:
	while ((c = input()) != '*' && c != 0);
//		putchar(c);

	if ((c1 = input()) != '/' && c != 0)
	{
		unput(c1);
		goto loop;
	}

	if (c != 0)
		putchar(c1);
	return 0;
}
